/*****************************************************
Chip type           : ATtiny13
Clock frequency     : 4,8 MHz
Memory model        : Tiny
External SRAM size  : 0
Data Stack size     : 16
*****************************************************/

#include <tiny13.h>
#include <delay.h>
#include <avr_eeprom.c>
          
#define red_on          PORTB.3 = 1;
#define red_off         PORTB.3 = 0;
#define green_on        PORTB.4 = 1;
#define green_off       PORTB.4 = 0;

#define reset_on        PORTB.0 = 0;
#define reset_off       PORTB.0 = 1; 

#define power_on        PORTB.2 = 0;
#define power_off       PORTB.2 = 1;

#define power           0xF0            //  power (POWER)
#define reset           0xD0            //  reset (BAND)
#define mono            0xB0            //   (MONO)

unsigned int tick = 0;          //  
unsigned char b_cnt = 0;        //  
bit start_cond = 0;             //  
bit repeat_cond = 0;            // 

unsigned int  addr = 0;         //2  
unsigned char cmd_1 = 0;        //  
unsigned char cmd_0 = 0;        //  
unsigned char cmd = 0;          // 
                                               
unsigned int j = 0;             //  

bit power_en = 1;               //  
bit auto = 1;                   // 
unsigned char repeat_cnt = 0;   // 

// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
        {
        green_on;
        TCNT0=0xC8;                                                             // 
        TCCR0B=0x02;
        
        if (tick >= 139 && tick < 150)                                          //   139  150 
                {
                start_cond = 1;                                                 //  
                repeat_cond = 0;                                                //  
                addr = cmd_1 = cmd_0 = 0;                                       //     
                }
        if (tick >= 116 && tick < 139)                                          //   116  138 
                {
                repeat_cond = 1;                                                //  
                start_cond = 0;                                                 //   
                TCNT0=0x00;
                }
        if (tick >= 22 && tick < 116 && start_cond)                             //   22  115 
                {
                ++b_cnt;                                                        // "1"    
                if (b_cnt < 17) addr = (addr << 1) + 1;                         // 2  - 
                if (b_cnt >= 16 && b_cnt < 25) cmd_1 = (cmd_1 << 1) + 1;        //  -  
                if (b_cnt >= 24) cmd_0 = (cmd_0 << 1) + 1;                      //  -  
                }
        if (tick >= 10 && tick < 22 && start_cond)                              //   10  21 
                {
                ++b_cnt;                                                        // "0"    
                if (b_cnt < 17) addr = (addr << 1);                             // -   
                if (b_cnt >= 16 && b_cnt < 25) cmd_1 = (cmd_1 << 1);
                if (b_cnt >= 24) cmd_0 = (cmd_0 << 1);
                }
        tick = 0;                                                               // 
        if (b_cnt == 32)                                                        //   4 
                {
                if ((cmd_1 + cmd_0) == 0xFF) cmd = cmd_1;                       //   
                        else cmd = 0;        
                b_cnt = 0;                                                      //  
                start_cond = 0;                                                 //  
                repeat_cond = 0;
                TCCR0B=0x00;                                                     // 
                TCNT0=0x00;
                }
        green_off;       
        }

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
        {
        TCNT0=0xC8;                                                             //  -  95
        if (++tick > 1200)                                                      //   1200 
                {
                TCCR0B=0x00;                                                    //    
                TCNT0=0x00;
                tick = 0;                                                       
                start_cond = 0;
                repeat_cond = 0;
                addr = cmd = 0;
                }
        }

// Declare your global variables here

void main(void)
{
// Declare your local variables here

// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Input/Output Ports initialization
// Port B initialization
// Func5=In Func4=Out Func3=Out Func2=Out Func1=In Func0=Out 
// State5=T State4=0 State3=0 State2=1 State1=P State0=1 
PORTB=0x07;
DDRB=0x1D;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 600,000 kHz
// Mode: Normal top=FFh
// OC0A output: Disconnected
// OC0B output: Disconnected
TCCR0A=0x00;
TCCR0B=0x00;
TCNT0=0x00;
OCR0A=0x00;
OCR0B=0x00;

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Falling Edge
// Interrupt on any change on pins PCINT0-5: Off
GIMSK=0x40;
MCUCR=0x02;
GIFR=0x40;

// Timer/Counter 0 Interrupt(s) initialization
TIMSK0=0x02;

// Analog Comparator initialization
// Analog Comparator: Off
ACSR=0x80;
ADCSRB=0x00;

// Global enable interrupts
#asm("sei")
        
        if (readEEPROM (10)) auto = 1;                  //   
                else auto = 0;
                                                                                               
        while (++j < 21 && power_en && auto)            //     -   
                {
                if (cmd == power || cmd == reset || cmd == 0x78) power_en = 0;          //     -  
                red_on;
                delay_ms(50);
                red_off;
                delay_ms(1500-(unsigned int)(j*75));
                }
        if (power_en && auto)        // -    , ..   
                {
                red_on;
                power_on;
                delay_ms(300);
                power_off;
                delay_ms(500);
                red_off;
                }

        while (1)
                {
                while (cmd == power)                                 //   Power  
                        {
                        if (repeat_cond) ++repeat_cnt;               //    (   )
                        while (repeat_cnt >= 8 && repeat_cond)       //    ,    
                                {
                                red_on;
                                power_on;
                                }
                        repeat_cond = 0;        
                        }
                repeat_cnt = 0;
                
                while (cmd == reset)                                 //   Band  
                        {                                            
                        if (repeat_cond) ++repeat_cnt;               //    (   )
                        if (repeat_cnt >= 10)                        // Reset    300   
                                {
                                red_on;
                                reset_on;
                                delay_ms(300);
                                reset_off;
                                red_off;
                                delay_ms(5000);
                                }
                        repeat_cond = 0;        
                        }
                repeat_cnt = 0;
                                
                while (cmd == mono)                                  //   Mono  
                        {
                        if (repeat_cond) ++repeat_cnt;               //    (   )
                        while (repeat_cnt >= 8 && repeat_cond)       //  
                                {
                                red_on;
                                auto = !auto;                        //  
                                writeEEPROM (10, auto);              //    
                                delay_ms(100);                       //  
                                red_off;
                                delay_ms(100);
                                red_on;
                                delay_ms(100);
                                red_off;
                                if (auto)                            //  ,    
                                        {
                                        delay_ms(100);
                                        red_on;
                                        delay_ms(100);
                                        red_off;
                                        }
                                delay_ms(3000);                      // 
                                break;
                                }
                        repeat_cond = 0;        
                        }
                repeat_cnt = 0;
                
                red_off;                                             // ,   .
                green_off;
                power_off;
                reset_off;
                };
}
